<?php
// no direct access
defined('_JEXEC') or die('Restricted access');

/* * ***********************************************************************
 * SQL App for Watchful
 * Copyright (c) 2006-2020 Watchful
 * ************************************************************************
 * Watchful.net is a web site monitoring and security service.
 *
 * Although Watchful.net is a paid service, this program is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version.
 *
 * This header must not be removed. Additional contributions/changes
 * may be added to this header as long as no information is deleted.
 * ************************************************************************
 * Get the latest version of this program at: https://watchful.net
 * *********************************************************************** */

require_once(JPATH_ADMINISTRATOR . '/components/com_watchfulli/classes/apps.php');
class plgWatchfulliAppsSqlApp extends watchfulliApps
{

    public function appMainProgram($oldValuesSerialized = null)
    {
        $this->setName("sqlApp");
        $this->setDescription("SQL App for Watchful - This App let you build simple SQL select query to check for custom values.");
        $this->db = & JFactory::getDbo();
        $debug = $this->params->get('debug', 0);

        // Get and validate table param
        $table = $this->params->get('table', '');
        if (!$this->validateTable($table))
        {
            if ($debug)
                $this->createAppAlert(1, "[DEBUG][SQLAPP] Table name $table is not valid");
            return $this;
        }

        // Get and validate field param
        $field = $this->params->get('field', '');
        if (!$this->validateField($table, $field))
        {
            if ($debug)
                $this->createAppAlert(1, "[DEBUG][SQLAPP] Field name $field is not valid");
            return $this;
        }

        // Get and validate function param
        $function = $this->params->get('function', '');
        if (!$this->validateFunction($function))
        {
            if ($debug)
                $this->createAppAlert(1, "[DEBUG][SQLAPP] SQL function $function is not valid");
            return $this;
        }

        // All parameters valid, get the value
        $newValue = $this->getValue($table, $field, $function);
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][SQLAPP] New value is: " . $newValue);

        if ($reference = $this->params->get('reference'))
        { // we check against a fixed value
            if ($debug)
                $this->createAppAlert(1, "[DEBUG][SQLAPP] Reference value is: " . $reference);
            if ($reference != $newValue)
            {
                $this->createAppAlert(1, "SQLAPP_VALUE_DIFFERENT_FROM_REFERENCE");
            }
            else
            {
                if ($debug)
                    $this->createAppAlert(1, "[DEBUG][SQLAPP] No changes");
            }
        }
        else
        { // we check against the old value
            $oldValue = $this->readExAppValue($oldValuesSerialized);
            if ($debug)
                $this->createAppAlert(1, "[DEBUG][SQLAPP] Old value is: " . $oldValue);
            if ($oldValue == null)
            {
                $this->createAppValue('sqlApp', $newValue);
                if ($debug)
                    $this->createAppAlert(1, "[DEBUG][SQLAPP] App executed for the first time.");
            }
            else if ($oldValue != $newValue)
            {
                $this->updateAppValue('sqlApp', $newValue);
                $this->createAppAlert(1, "SQLAPP_VALUE_CHANGED");
            }
            else
            {
                if ($debug)
                    $this->createAppAlert(1, "[DEBUG][SQLAPP] No changes");
            }
        }
        return $this;
    }

    private function validateTable($table)
    {
        if ($table == '')
            return false;
        $this->db->setQuery("SELECT 1 FROM " . $this->db->quoteName($table));
        return( $this->db->loadResult());
    }

    private function validateField($table, $field)
    {
        if (!method_exists($this->db, 'getTableFields')) {
            $available_fields = $this->db->getTableColumns($table);
            return !empty($available_fields[$field]);
        }
        $available_fields = $this->db->getTableFields($table);
        $available_fields = empty($available_fields[$table]) ? array() : $available_fields[$table];
        return( isset($available_fields[$table][$field]));
    }

    private function validateFunction($function)
    {
        $available_functions = array('MAX', 'MIN', 'SUM');
        return( in_array($function, $available_functions));
    }

    private function getValue($table, $field, $function)
    {
        $query = "SELECT $function($field) FROM $table";
        $this->db->setQuery($query);
        return( $this->db->loadResult() );
    }

}
